"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = require("path");

var _toolLib = require("@kettil/tool-lib");

var _chalk = require("chalk");

var _istanbulLibCoverage = require("istanbul-lib-coverage");

var _istanbulLibReport = require("istanbul-lib-report");

var _istanbulReports = require("istanbul-reports");

var _existFiles = _interopRequireDefault(require("../../cmd/existFiles"));

var _readFile = _interopRequireDefault(require("../../cmd/readFile"));

var _writeFile = _interopRequireDefault(require("../../cmd/writeFile"));

var _messageError = _interopRequireDefault(require("../../errors/messageError"));

var _jest = require("./jest");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const coverageFileName = 'coverage-final.json';
const coverageSuccessKeys = new Set(['high', 'medium']);
const watermarkKeys = ['statements', 'functions', 'branches', 'lines'];

const getWatermark = watermark => {
  if ((0, _toolLib.isArray)(watermark)) {
    return watermark;
  }

  if (typeof watermark === 'number') {
    return [watermark, watermark];
  }

  return [90, 99];
};

const getErrorMessage = (key, watermarks, coverageSummary) => {
  const limit = Math.min(...getWatermark(watermarks[key]));
  const value = coverageSummary[key].pct;
  return (0, _chalk.red)(`Coverage threshold for ${(0, _chalk.bold)(key)} (${limit}%) not met: ${value}%`);
};

const coverage = async (argv, {
  projects,
  watermarks = {}
}) => {
  const coverageFiles = await (0, _existFiles.default)(projects.map(project => (0, _path.join)((0, _jest.getCoverageFolder)(argv.cwd, [project]), coverageFileName)));
  const coveragePath = (0, _jest.getCoverageFolder)(argv.cwd, []);
  const coverageMap = (0, _istanbulLibCoverage.createCoverageMap)({});
  const coverages = await Promise.all(coverageFiles.map(file => (0, _readFile.default)(file, true)));
  coverages.forEach(coverageData => coverageMap.merge(coverageData));
  await (0, _writeFile.default)((0, _path.join)(coveragePath, coverageFileName), JSON.stringify(coverageMap));
  const context = (0, _istanbulLibReport.createContext)({
    coverageMap,
    dir: coveragePath,
    // The summarizer to default to (may be overridden by some reports)
    // values can be nested/flat/pkg. Defaults to 'pkg'
    defaultSummarizer: 'pkg',
    watermarks: watermarkKeys.reduce((data, key) => Object.assign(data, {
      [key]: getWatermark(watermarks[key])
    }), {})
  }); // create reports

  (0, _istanbulReports.create)('lcovonly', {}).execute(context);
  (0, _istanbulReports.create)('json-summary', {}).execute(context);
  (0, _istanbulReports.create)('text-summary', {}).execute(context);

  if (!argv.ci) {
    (0, _istanbulReports.create)('html', {}).execute(context);
  }

  const coverageSummary = coverageMap.getCoverageSummary();
  const coverageErrors = watermarkKeys.filter(key => !coverageSuccessKeys.has(context.classForPercent(key, coverageSummary[key].pct)));

  if (coverageErrors.length > 0) {
    throw new _messageError.default(coverageErrors.map(key => getErrorMessage(key, watermarks, coverageSummary)).join('\n'));
  }
};

var _default = coverage;
exports.default = _default;